/* Copyright 2010 Fredrik Wikstrom. All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
**
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
**
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
*/

#include "cddapatch.h"
#include <proto/dos.h>

/**********************************************************
**
** The following function saves the variable name passed in 
** 'varname' to the ENV(ARC) system so that the application 
** can become AmiUpdate aware.
**
**********************************************************/

void SetAmiUpdateENVVariable (const char *varname) {
	/* AmiUpdate support code */
	BPTR lock;
  
	/* obtain the lock to the home directory */
	if ((lock = IDOS->GetProgramDir())) {
		TEXT progpath[2048];
		TEXT varpath[1024] = "AppPaths";

		/*
		get a unique name for the lock, 
		this call uses device names,
		as there can be multiple volumes 
		with the same name on the system
		*/

		if (IDOS->DevNameFromLock(lock, progpath, sizeof(progpath), DN_FULLPATH)) {
			APTR oldwin;

			/* stop any "Insert volume..." type requesters */
			oldwin = IDOS->SetProcWindow((APTR)-1);
        
			/* 
			finally set the variable to the 
			path the executable was run from 
			don't forget to supply the variable 
			name to suit your application 
			*/

			IDOS->AddPart(varpath, varname, 1024);
			IDOS->SetVar(varpath, progpath, -1, GVF_GLOBAL_ONLY|GVF_SAVE_VAR);

			/* turn requesters back on */
			IDOS->SetProcWindow(oldwin);
		}
	}
}
